angular.module('rio')
        .config(function ($stateProvider, $urlRouterProvider, erkStateSmootherProvider) {

            // Ionic uses AngularUI Router which uses the concept of states
            // Learn more here: https://github.com/angular-ui/ui-router
            // Set up the various states which the app can be in.
            // Each state's controller can be found in controllers.js

            // if none of the states are matched, use this as the fallback

            /*********************************************************************
             * Inserita una forzatura per utilizzare sempre l'italiano           *
             *********************************************************************/
            //$urlRouterProvider.otherwise('/language');
            $urlRouterProvider.otherwise('/landing');

            // erkStateSmootherProvider convert nested states into plain states
            // don't forget .parse() at the end of cancatenated .state()s
            // otherwise $stateProvider will not be setted with new plain beautiful-transitive states.
            erkStateSmootherProvider
                    // this state is used to ensure evrything is loaded (angular + ionic + cordova plugins)
                    // + it's eventually used to preload assets and data
                    .state('landing', {
                        url: '/landing',
                        templateUrl: 'views/landing/landing.html',
                        controller: 'LandingCtrl'
                    })

                    .state('landing.credits', {
                        url: '/credits',
                        //templateUrl: 'views/landing/credits/credits.html',

                        templateUrl: function (templateProvider) {
                            //selectedLanguage = angular.fromJson(window.localStorage['selectedLanguage']).lang;
                            selectedLanguage = "it";
                            return 'views/landing/credits/credits.' + selectedLanguage + '.html';
                        },

                        controller: 'CreditsCtrl'
                    })
                    .state('welcome', {
                        url: '/welcome',
                        templateUrl: 'views/welcome/Welcome.html',
                        controller: 'WelcomeCtrl',
                        resolve: {
                            profile: function (rioProfileService) {
                                return rioProfileService.get().then(function (profile) {
                                    return profile;
                                });
                            },
                            users: function (rioUsersService) {
                                return rioUsersService.all();
                            }
                        }
                    })
                    .state('welcome.science', {
                        url: '/science',
                        templateUrl: 'views/science/home/home.html',
                        controller: 'ScienceCtrl'
                    })
                    .state('welcome.science.faq', {
                        url: '/faq',
                        templateUrl: 'views/science/faq/faq.html',
                        controller: 'FaqCtrl',
                        resolve: {
                            faq: function ($erkJsonDataBase, $translate) {
                                var lang = $translate.use();
                                return $erkJsonDataBase.data('faq_' + lang).all();
                            }
                        }
                    })
                    .state('welcome.science.lineeguida', {
                        url: '/faq',
                        templateUrl: 'views/science/lineeguida/lineeguida.html',
                        controller: 'LineeguidaCtrl'
                    })
                    .state('welcome.science.glossary', {
                        url: '/glossary',
                        templateUrl: 'views/science/glossary/glossary.html',
                        controller: 'GlossaryCtrl',
                        resolve: {
                            glossary: function ($erkJsonDataBase, $translate) {
                                var lang = $translate.use();
                                return $erkJsonDataBase.data('glossary_' + lang).all();
                            }
                        }
                    })

                    // setup a state to resolve profile, users and exercises informations
                    // it will select the app entry point.
                    .state('root', {
                        url: '/',
                        template: '<ion-nav-view></ion-nav-view>',
                        resolve: {
                            profile: function (rioProfileService) {
                                return rioProfileService.get().then(function (profile) {
                                    return profile;
                                });
                            },
                            users: function (rioUsersService) {
                                return rioUsersService.all();
                            },
                            videos: function (vmVideoService) {
                                return vmVideoService.all();
                            },
                            luoghi: function (vmLuoghiService) {
                                return vmLuoghiService.all();
                            },
                            reports: function (vmVideoReportService) {
                                return vmVideoReportService.all();
                            }
                        }
                    })
                    .state('language', {
                        url: '/language',
                        templateUrl: 'views/chooselang/chooselang.html',
                        controller: 'ChooseLangCtrl',
                        params: {
                            fromPage: null
                        }
                    })
                    .state('root.config', {
                        url: '/config',
                        abstract: true,
                        template: '<ion-nav-view></ion-nav-view>'
                    })
                    .state('root.config.profile', {
                        url: '/profile',
                        templateUrl: 'views/config/profile/profile.html',
                        controller: 'ConfigProfileCtrl'
                    })
                    .state('root.accounts', {
                        url: '/accounts',
                        abstract: true,
                        template: '<ion-nav-view></ion-nav-view>'
                    })
                    
                    .state('root.accounts.start', {
                        url: '/start',
                        templateUrl: 'views/accounts/start/start.html',
                        controller: 'AccountsStartCtrl'
                    })
                    .state('root.accounts.userlist', {
                        url: '/userlist',
                        templateUrl: 'views/accounts/userList/userList.html',
                        controller: 'UserListCtrl'
                    })
                    .state('root.accounts.help', {
                        url: '/help',
                        templateUrl: 'views/accounts/help/Help.html',
                        controller: 'HelpCtrl'
                    })
                    

                    .state('root.mainmenu', {
                        url: '/mainmenu:activeId',
                        templateUrl: 'views/mainmenu/mainMenu.html',
                        controller: 'MainMenuCtrl',
                        resolve: {
                            activeId: function ($stateParams, rioProfileService) {
                                if ($stateParams.activeId.length <= 0) {
                                    return rioProfileService.get().then(function (profile) {
                                        return angular.isDefined(profile) && angular.isDefined(profile.id_activeuser) ? profile.id_activeuser : -1;
                                    });
                                } else {
                                    return parseInt($stateParams.activeId);
                                }
                            },
                            activeUser: function (activeId, rioActiveUserService) {
                                return rioActiveUserService.activeUser(activeId);
                            },
                            userVideos: function (activeId, vmVideoService) {
                                return vmVideoService.getForUser(activeId);
                            },
                            videoData: function ($http, $erkJsonDataBase) {
                                return {connectionActive: false, data: $erkJsonDataBase.data('videoData').all()};
                                
                                /*return $http({
                                    method: 'GET',
                                    url: VIDEO_LIST_URL,
                                    timeout: 3000
                                }).then(function successCallback(response) {
                                    return {connectionActive: true, data: response.data};
                                }, function errorCallback(response) {
                                    if($erkJsonDataBase.data('videoData')) {
                                        return {connectionActive: false, data: $erkJsonDataBase.data('videoData').all()};
                                    } else {
                                        return {connectionActive: false, data: []};
                                    }
                                });*/
                            }
                        }
                    })
                    .state('root.mainmenu.help', {
                        url: '/menuhelp:activeId',
                        templateUrl: 'views/mainmenu/menuhelp/menuhelp.html',
                        controller: 'menuhelpCtrl'
                    })
                    .state('root.metrics', {
                        url: '/metrics:activeId',
                        templateUrl: 'views/andamento/metricsMenu.html',
                        controller: 'MetricsMenuCtrl',
                        resolve: {
                            activeId: function ($stateParams, rioProfileService) {
                                if ($stateParams.activeId.length <= 0) {
                                    return rioProfileService.get().then(function (profile) {
                                        return angular.isDefined(profile) && angular.isDefined(profile.id_activeuser) ? profile.id_activeuser : -1;
                                    });
                                } else {
                                    return parseInt($stateParams.activeId);
                                }
                            },
                            activeUser: function (activeId, rioActiveUserService) {
                                return rioActiveUserService.activeUser(activeId);
                            }
                        }
                    })

                    .state('root.metrics.stats', {
                        url: '/stats:activeId:type',
                        templateUrl: 'views/andamento/stats/stats.html',
                        controller: 'StatsCtrl',
                        resolve: {
                            type: function ($stateParams) {
                                if ($stateParams.type.length > 0) {
                                    return parseInt($stateParams.type);
                                } else {
                                    return 1;
                                }
                            }
                        }
                    })
                    .state('root.metrics.stats.videos', {
                        url: '/stats-:activeId-:type',
                        templateUrl: 'views/andamento/stats/videos/statsDetailVideoList.html',
                        controller: 'statsDetailVideoListCtrl',
                        resolve: {
                            userVideoReports: function (activeId, vmVideoService) {
                                //TODO: Get only active user videos
                                return vmVideoService.getForUserWithReport(activeId);
                            }
                        }
                    })

                    .state('root.metrics.stats.videos.report', {
                        url: '/reports:activeId:videoId:type',
                        templateUrl: 'views/andamento/stats/videos/statsDetailVideoReport.html',
                        controller: 'statsDetailVideoReportCtrl',
                        resolve: {
                            video: function ($stateParams, vmVideoService) {
                                return vmVideoService.getFromUuid($stateParams.videoId);
                            },
                            report: function (activeId, $stateParams, vmVideoReportService) {
                                return vmVideoReportService.getFromVideo($stateParams.videoId, activeId);
                            }
                        }
                    })

                    .state('root.metrics.communications', {
                        url: '/communications:activeId',
                        templateUrl: 'views/andamento/communications/communications.html',
                        controller: 'CommunicationsCtrl',
                        resolve: {
                            communications: function (activeId, vmMetricsCommunicationService) {
                                return vmMetricsCommunicationService.getForUser(activeId, true);
                            }
                        }
                    })
                    .state('root.metrics.communications.detail', {
                        url: '/communication-detail:activeId-:commId',
                        templateUrl: 'views/andamento/communications/detail/detail.html',
                        controller: 'CommunicationDetailCtrl',
                        resolve: {
                            commId: function ($stateParams) {
                                return parseInt($stateParams.commId);
                            },
                            communication: function ($stateParams, vmMetricsCommunicationService) {
                                return vmMetricsCommunicationService.get($stateParams.commId, true);
                            },
                            originalCommunication: function ($stateParams, vmMetricsCommunicationService) {
                                return vmMetricsCommunicationService.get($stateParams.commId, false);
                            }
                        }
                    })
                   
                    .state('root.video', {
                        url: '/video',
                        abstract: true,
                        template: '<ion-nav-view></ion-nav-view>'
                    })
                    .state('root.video.list', {
                        url: '/videolist:activeId:newVideo',
                        templateUrl: 'views/video/videoList.html',
                        controller: 'VideoListCtrl',
                        cache: false,
                        resolve: {
                            activeId: function ($stateParams, rioProfileService) {
                                if ($stateParams.activeId.length <= 0) {
                                    return rioProfileService.get().then(function (profile) {
                                        return angular.isDefined(profile) && angular.isDefined(profile.id_activeuser) ? profile.id_activeuser : -1;
                                    });
                                } else {
                                    return parseInt($stateParams.activeId);
                                }
                            },
                            activeUser: function (activeId, rioActiveUserService) {
                                return rioActiveUserService.activeUser(activeId);
                            },
                            videos: function (activeId, vmVideoService) {
                                return vmVideoService.getForUser(activeId);
                            },
                            newVideo: function ($stateParams) {
                                if ($stateParams.newVideo.length <= 0) {
                                    return -1;
                                } else {
                                    return parseInt($stateParams.newVideo);
                                }
                            },
                            addedVideo: function (newVideo, vmVideoService) {
                                return vmVideoService.get(newVideo);
                            }
                        }
                    })

                    .state('root.video.detail', {
                        url: '/detail:videoId:activeId',
                        templateUrl: 'views/video/detail/videoDetail.html',
                        cache: false,
                        controller: 'VideoDetailCtrl',
                        resolve: {
                            videoSelezionato: function ($stateParams, vmVideoService) {
                                return vmVideoService.get($stateParams.videoId);
                            },
                            activeId: function ($stateParams, rioProfileService) {
                                if ($stateParams.activeId.length <= 0) {
                                    return rioProfileService.get().then(function (profile) {
                                        return angular.isDefined(profile) && angular.isDefined(profile.id_activeuser) ? profile.id_activeuser : -1;
                                    });
                                } else {
                                    return parseInt($stateParams.activeId);
                                }
                            }
                        }
                    })

                    .state('root.video.prompting', {
                        url: '/prompting:videoId',
                        templateUrl: 'views/video/prompting/videoPrompting.html',
                        cache: false,
                        controller: 'VideoPromptingCtrl',
                        resolve: {
                            videoSelezionato: function ($stateParams, vmVideoService) {
                                return vmVideoService.get($stateParams.videoId);
                            }
                        }
                    })
                    .state('root.video.publicdetail', {
                        url: '/videopublicdetail:videoId',
                        templateUrl: 'views/video/publicdetail/videoDetail.html',
                        controller: 'VideoPublicDetailCtrl',
                        resolve: {
                            video: function ($stateParams, vmVideoService) {
                                return vmVideoService.get($stateParams.videoId);
                            }
                        }
                    })

                    .state('root.video.monitoring', {
                        url: '/videomonitoring:videoId:activeId:uuid',
                        templateUrl: 'views/video/monitoring/monitoring.html',
                        controller: 'MonitoringCtrl',
                        resolve: {
                            video: function ($stateParams, vmVideoService) {
                                return vmVideoService.get($stateParams.videoId);
                            },
                            report: function ($stateParams, activeId, vmVideoReportService) {
                                return vmVideoReportService.getFromVideo($stateParams.uuid, activeId);
                            },
                            activeId: function ($stateParams, rioProfileService) {
                                if ($stateParams.activeId.length <= 0) {
                                    return rioProfileService.get().then(function (profile) {
                                        return angular.isDefined(profile) && angular.isDefined(profile.id_activeuser) ? profile.id_activeuser : -1;
                                    });
                                } else {
                                    return parseInt($stateParams.activeId);
                                }
                            },
                            activeUser: function (activeId, rioActiveUserService) {
                                return rioActiveUserService.activeUser(activeId);
                            }
                        }
                    })
                    .state('root.video.monitoring.params', {
                        url: '/parameters:videoId:userId',
                        templateUrl: 'views/video/monitoring/params/params.html',
                        controller: 'ParamsCtrl',
                        resolve: {
                            report: function ($stateParams, vmVideoReportService) {
                                return vmVideoReportService.getFromVideo($stateParams.videoId, $stateParams.userId).then(
                                        function (report) {
                                            report.userId = $stateParams.userId;
                                            return report;
                                        }
                                );
                            }
                        }
                    })
                    .state('root.video.monitoring.help', {
                        url: '/monitoringhelp:activeId',
                        templateUrl: 'views/video/monitoring/monitoringhelp/monitoringhelp.html',
                        controller: 'monitoringhelpCtrl'
                    })
                    
                    .state('root.video.review', {
                        url: '/videomonitoring:videoId',
                        templateUrl: 'views/video/rating/ratingview.html',
                        controller: 'RatingCtrl',
                        resolve: {
                            video: function ($stateParams, vmVideoService) {
                                return vmVideoService.get($stateParams.videoId);
                            }
                        }
                    })

                    //area bambino

                    .state('root.bambino', {
                        url: '/bambino:activeId-:mode',
                        templateUrl: 'views/bambino/areaBambino.html',
                        controller: 'AreaBambinoCtrl',
                        resolve: {
                            activeId: function ($stateParams, rioProfileService) {
                                if ($stateParams.activeId.length <= 0) {
                                    return rioProfileService.get().then(function (profile) {
                                        return angular.isDefined(profile) && angular.isDefined(profile.id_activeuser) ? profile.id_activeuser : -1;
                                    });
                                } else {
                                    return parseInt($stateParams.activeId);
                                }
                            },
                            activeUser: function (activeId, rioActiveUserService) {
                                return rioActiveUserService.activeUser(activeId);
                            },
                            area: function (activeId, vmBambinoAreaService) {
                                return vmBambinoAreaService.get(activeId);
                            },
                            mode: function ($stateParams) {
                                return $stateParams.mode;
                            }
                        }
                    })
                    .state('root.bambino.video', {
                        url: '/video:activeId',
                        templateUrl: 'views/bambino/video/videoList.html',
                        controllerProvider: function () {
                            return (document.body.className.match('bambino') != null) ? 'RealChildVideoListCtrl' : 'ChildVideoListCtrl';
                        },
                        controllerAs: "ChildVideoListCtrl",
                        resolve: {
                            items: function (activeId, vmVideoService) {
                                return vmVideoService.getVideoInBambino(activeId);
                            }
                        }
                    })
                    .state('root.bambino.video.add', {
                        url: '/video-add:activeId',
                        templateUrl: 'views/bambino/video/add/videoList.html',
                        controller: 'ChildVideoListPickerCtrl',
                        resolve: {
                            addableItems: function (activeId, vmVideoService) {
                                return vmVideoService.getVideoNotInBambino(activeId);
                            }
                        }
                    })
                    .state('root.bambino.luoghi', {
                        url: '/luoghi:activeId',
                        templateUrl: 'views/bambino/luoghi/luoghiList.html',
                        controller: 'ChildLuoghiListCtrl',
                        resolve: {
                            items: function (activeId, vmLuoghiService) {
                                return vmLuoghiService.getInBambino(activeId);
                            }
                        }
                    })
                    .state('root.bambino.luoghi.add', {
                        url: '/luoghi-add:activeId',
                        templateUrl: 'views/bambino/luoghi/add/luoghiList.html',
                        controller: 'ChildLuoghiListPickerCtrl',
                        resolve: {
                            addableItems: function (activeId, vmLuoghiService) {
                                return vmLuoghiService.getNotInBambino(activeId);
                            }
                        }
                    })

                    .state('root.luoghi', {
                        url: '/luoghi',
                        abstract: true,
                        template: '<ion-nav-view></ion-nav-view>'
                    })

                    //luoghi
                    .state('root.luoghi.list', {
                        url: '/luoghilist:activeId:newLuogo',
                        templateUrl: 'views/luoghi/luoghiList.html',
                        controller: 'LuoghiListCtrl',
                        resolve: {
                            activeId: function ($stateParams, rioProfileService) {
                                if ($stateParams.activeId.length <= 0) {
                                    return rioProfileService.get().then(function (profile) {
                                        return angular.isDefined(profile) && angular.isDefined(profile.id_activeuser) ? profile.id_activeuser : -1;
                                    });
                                } else {
                                    return parseInt($stateParams.activeId);
                                }
                            },
                            luoghi: function (activeId, vmLuoghiService) {
                                return vmLuoghiService.getForUser(activeId);
                            },
                            activeUser: function (activeId, rioActiveUserService) {
                                return rioActiveUserService.activeUser(activeId);
                            },
                            newLuogo: function ($stateParams) {
                                if ($stateParams.newLuogo.length <= 0) {
                                    return -1;
                                } else {
                                    return parseInt($stateParams.newLuogo);
                                }
                            },
                            addedLuogo: function (newLuogo, vmLuoghiService) {
                                return vmLuoghiService.get(newLuogo);
                            },
                            videos: function (activeId, vmVideoService) {
                                return vmVideoService.getForUser(activeId);
                            }
                        }
                    })

                    .state('root.luoghi.detail', {
                        url: '/luoghidetail:luogoId:mode',
                        templateUrl: 'views/luoghi/detail/luogoDetail.html',
                        controller: 'LuogoDetailCtrl',
                        resolve: {
                            luogo: function ($stateParams, vmLuoghiService) {
                                return vmLuoghiService.get($stateParams.luogoId).then(
                                        function (luogo) {
                                            return luogo;
                                        }
                                );
                            },
                            videos: function (luogo, vmVideoService) {
                                //TODO: Get only active user videos
                                return vmVideoService.getForUser(luogo.userId);
                            },
                            mode: function ($stateParams) {
                                return $stateParams.mode;
                            }
                        }
                    })
                    
                    .state('root.luoghi.videodetail', {
                        url: '/detail:videoId:activeId',
                        templateUrl: 'views/video/detail/videoDetail.html',
                        cache: false,
                        controller: 'VideoDetailCtrl',
                        resolve: {
                            videoSelezionato: function ($stateParams, vmVideoService) {
                                return vmVideoService.get($stateParams.videoId);
                            },
                            activeId: function ($stateParams, rioProfileService) {
                                if ($stateParams.activeId.length <= 0) {
                                    return rioProfileService.get().then(function (profile) {
                                        return angular.isDefined(profile) && angular.isDefined(profile.id_activeuser) ? profile.id_activeuser : -1;
                                    });
                                } else {
                                    return parseInt($stateParams.activeId);
                                }
                            }
                        }
                    });


            erkStateSmootherProvider.parse();
        });
